/* This file was generated by syncqt. */
#ifndef QT_QTDEVICEDISCOVERYSUPPORT_VERSION_H
#define QT_QTDEVICEDISCOVERYSUPPORT_VERSION_H

#define QTDEVICEDISCOVERYSUPPORT_VERSION_STR "6.11.1"

#define QTDEVICEDISCOVERYSUPPORT_VERSION 0x0601101

#endif // QT_QTDEVICEDISCOVERYSUPPORT_VERSION_H
